package util.multicast;

import util.misc.Misc;
import util.models.BoundedBuffer;

public class AMessageSenderRunnable implements Runnable {
	BoundedBuffer<ASentMessage> outputMessageQueue;
	MessageDelayer sessionManagerClient;
	MulticastGroup localMulticastGroup;
	MulticastGroup multicastGroup;
	SessionManager sessionManager;
	Session session;
	int delayToServer;
	public AMessageSenderRunnable(BoundedBuffer<ASentMessage> theMessageQueue,
			MessageDelayer theSessionManagerClient,
			SessionManager theSessionManager
			) {
		outputMessageQueue = theMessageQueue;
		sessionManagerClient = theSessionManagerClient;
		sessionManager = theSessionManager;
		//multicastGroup = theMulticastGroup; // in case it is local
		//multicastServer = theMulticastServer;
	}
	public void setLocalMulticastGroup(MulticastGroup theMulticastGroup) {
		localMulticastGroup = theMulticastGroup;
		multicastGroup = localMulticastGroup;
	}
	void sleep (long delay) throws InterruptedException {
		if (delay > 0)
			Thread.sleep(delay);
	}
	void sleepIfRemoteMulticastGroup(long delay) throws InterruptedException {
		if (localMulticastGroup != null)
			sleep(delay);
	}
	public void run() {
		
		while (true) {
		try {
			ASentMessage message = outputMessageQueue.get();
			Object[] args = message.args;
			//long delay = multicastClient.getDelayToServer() - (System.currentTimeMillis() - message.timeStamp);
			long delay = sessionManagerClient.calculateDelay(message.timeStamp);

//			if (delay > 0)
//				Thread.sleep(delay);
			switch (message.multicastMessageType) {
			case Join:
				sleep(delay);
				JoinReturnValue joinReturnValue = sessionManager.join((String) args[0], (String) args[1], (String) args[2], (MessageReceiver) args[3]);
				sleep(delay);// take into account delay for return value
				session = joinReturnValue.session;	
				if (multicastGroup == null)
					multicastGroup = joinReturnValue.multicastGroup;
				sessionManagerClient.setClients(joinReturnValue.clients);
				sessionManagerClient.setSerializedMulticastGroups(joinReturnValue.serializedMulticastGroups);
				break;
			case Leave:
				sleep(delay);
				session.leave((String) args[0], (MessageReceiver) args[1], null);
				break;
			case Others:
				sleepIfRemoteMulticastGroup(delay);
				multicastGroup.toOthers(args[0], (String) args[1], (MessageReceiver) args[2]);				
				break;
			case All:
				sleepIfRemoteMulticastGroup(delay);
				multicastGroup.toAll(args[0], (String) args[1], (MessageReceiver) args[2]);
				break;
			case Host:
				sleepIfRemoteMulticastGroup(delay);
				multicastGroup.toHost((String) args[0],args[1], (String) args[2], (MessageReceiver) args[3]);
				break;
			case Hosts:
				sleepIfRemoteMulticastGroup(delay);
				multicastGroup.toHosts((String[]) args[0],args[1], (String) args[2], (MessageReceiver) args[3]);
				break;
			
			}
			
			
		} catch (Exception e) {
			e.printStackTrace();
		}
		}

	}

}
