package util.multicast;

import java.io.Serializable;
import java.rmi.RemoteException;
import java.rmi.registry.Registry;
import java.rmi.server.UnicastRemoteObject;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import util.misc.Message;
import util.misc.Misc;
import util.models.BoundedBuffer;
@util.annotations.StructurePattern(util.annotations.StructurePatternNames.BEAN_PATTERN)
public abstract class ASessionManagerClient extends ASessionListenable implements MessageSender, MessageReceiver, MessageDelayer, Serializable {
	//MessageReceiver exportedReference;
	MessageReceiver exportedReference;
	String clientName;
	String sessionName;
	String applicationName;
	MulticastGroup multicastServer;
	SessionManager sessionManager;
	//Session session;
	List<ReceivedMessageListener> receivedMessageListeners = new ArrayList();
	BoundedBuffer<ASentMessage> outputMessageQueue;
	AMessageSenderRunnable messageSender;
	Thread messageSenderThread;
	Map <MessageReceiver, String>  clients = new HashMap();
	SerializedMulticastGroups serializedMulticastGroups;
	BoundedBuffer<AReceivedMessage> inputMessageQueue;
	AMessageReceiverRunnable messageReceiver;
	Thread messageReceiverThread;
	int delayToServer, delayVariation;
	public ASessionManagerClient(String theServerHost, String theSessionName, String theApplicationName, String theClientName) {
		join(theServerHost, theSessionName, theApplicationName, theClientName);
	}
	public  ASessionManagerClient  (String[] args) {
		if (args.length < 4) {
			Message.fatalError("Please supply server host name, session Name, and clientName as main argument");			
		}
		join(args[0], args[1], args[2], args[3]);		
	}
	SessionManager getSessionManagerHandle(String serverHost) {
		try {
			Registry registry = Misc.getRegistry(serverHost);
			return (SessionManager) registry.lookup(SessionManager.SESSION_MANAGER_NAME);
		} catch (Exception e) {
			e.printStackTrace();
			Message.fatalError("Could not lookup multicast server on host:" + serverHost);
			return null;
		}
	}
//	MulticastServer getMulticastServerHandle() {
//		try {
//			multicastServer = (MulticastServer) sessionManager;
//			return multicastServer;
//		} catch (Exception e) {
//			e.printStackTrace();
//			Message.fatalError("Session manager is not multicast server");
//			return null;
//		}
//	}
	MessageReceiver generateRemoteProxyToSelf() {
		try {
			return (MessageReceiver) UnicastRemoteObject.exportObject((MessageReceiver) this, 0);
		} catch (Exception e) {
			e.printStackTrace();
			return null;
		}	
	}
	void createOutputBufferAndThread() {
		outputMessageQueue = new BoundedBuffer();
		messageSender = new AMessageSenderRunnable(outputMessageQueue, this, sessionManager);
		messageSenderThread = new Thread(messageSender);
		messageSenderThread.start();
//		Object[] args = {sessionName, applicationName, clientName, exportedReference};
//		outputMessageQueue.put(new ASentMessage(SentMessageType.Join, args));		
	}
	void createInputBufferAndThread() {
		inputMessageQueue = new BoundedBuffer();
		messageReceiver = new AMessageReceiverRunnable(inputMessageQueue, this);
		messageReceiverThread = new Thread(messageReceiver);
		messageReceiverThread.start();
	}
	
	@Override
	public void join(String serverHost, String theSessionName, String theApplicationName, String theClientName) {
		try {
			clientName = theClientName;
			sessionManager = getSessionManagerHandle(serverHost);
			sessionName = theSessionName;
			applicationName = theApplicationName;
			//session = sessionManager.getOrCreateSession(theSessionName);
			//multicastServer = getMulticastServerHandle();
			exportedReference = generateRemoteProxyToSelf();
			createOutputBufferAndThread();
			createInputBufferAndThread();
			Object[] args = {sessionName, applicationName, clientName, exportedReference};
			outputMessageQueue.put(new ASentMessage(SentMessageType.Join, args));		
		} catch (Exception e) {
			e.printStackTrace();
			Message.fatalError("Could not lookup multicast server on host:" + serverHost);
		}		
	}

	@Override
	public void leave() {
		// TODO Auto-generated method stub
		
	}

//	@Override
//	public void toHost(String hostName, Object object) {
//		// TODO Auto-generated method stub
//		
//	}
//
//	@Override
//	public void toHosts(String[] hostNames, Object object) {
//		// TODO Auto-generated method stub
//		
//	}
//
//	@Override
//	public void toOthers(Object object) {
//		try {
//			Object[] args = {object, clientName, exportedReference};
//			ASentMessage message = new ASentMessage(SentMessageType.Others, args) ;
//			outputMessageQueue.put(message);
//			//server.toOthers(object, clientName, exportedReference);
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//	}
//	@Override
//	public void toAll(Object object) {
//		try {
//			Object[] args = {object, clientName, exportedReference};
//			ASentMessage message = new ASentMessage(SentMessageType.All, args) ;
//			outputMessageQueue.put(message);
//		} catch (Exception e) {
//			e.printStackTrace();
//		}
//	}
	@Override
	public synchronized void newObject(String clientName, MessageReceiver client, Object value) {
		//System.out.println("New Object: " + value + " from: " + clientName);		
//		for (ReceivedMessageListener listener: receivedMessageListeners) {
//			listener.messageReceived(value, clientName);
//		}
		AReceivedMessage receivedMesage = new AReceivedMessage(ReceivedMessageType.NewObject, clientName, client, null, value);
		inputMessageQueue.put(receivedMesage);
	}
	public synchronized void delayedNewObject(String clientName, Object value) {
		//System.out.println("New Object: " + value + " from: " + clientName);		
		for (ReceivedMessageListener listener: receivedMessageListeners) {
			listener.objectReceived(value, clientName);
		}
	}
	public synchronized void delayedUserJoined(String theClientName, MessageReceiver theClient, String theApplicationName) {
		//System.out.println("New Object: " + value + " from: " + clientName);			
		for (ReceivedMessageListener listener: receivedMessageListeners) {
			listener.userJoined(theClientName, theApplicationName);
		}
		if (applicationName.equals(theApplicationName))
			clients.put(theClient, theClientName);
			
	}
	public synchronized void delayedUserLeft(String theClientName, MessageReceiver theClient, String theApplicationName) {
		//System.out.println("New Object: " + value + " from: " + clientName);		
		for (ReceivedMessageListener listener: receivedMessageListeners) {
			listener.userLeft(theClientName, theApplicationName);
		}
		if (applicationName.equals(theApplicationName))
			clients.remove(theClientName);
	}
	public void userJoined(String clientName, MessageReceiver client, String theApplicationName) throws RemoteException {
		AReceivedMessage receivedMesage = new AReceivedMessage(ReceivedMessageType.ClientJoined, clientName, client, theApplicationName, null);
		inputMessageQueue.put(receivedMesage);		
		
	}	
	public void userLeft(String theClientName, MessageReceiver theClient, String theApplicationName) throws RemoteException {
		AReceivedMessage receivedMesage = new AReceivedMessage(ReceivedMessageType.ClientLeft, theClientName, theClient, theApplicationName, null);
		inputMessageQueue.put(receivedMesage);		
	}
	public int getMinimumDelayToServer() {
		return delayToServer;
	}	
	public void setMinimumDelayToServer(int theDelay) {
		delayToServer = theDelay;
	}
	public int getDelayVariation() {
		return delayVariation;
	}
	public void setDelayVariation(int theDelay) {
		delayVariation = theDelay;
	}
	@Override
	public String getClientName() {
		return clientName;
	}
	@Override
	public void addReceivedMessageListener(ReceivedMessageListener listener) {
		if (receivedMessageListeners.contains(listener))
			return;
		receivedMessageListeners.add(listener);
	}
	@Override
	public void removeReceivedMessageListener(ReceivedMessageListener listener) {
		receivedMessageListeners.remove(listener);
	}
	public long calculateDelay (long messageTime) {
		double random = Math.random();
		long randomDelay = Math.round(getDelayVariation()*random);
		long actualDelay = randomDelay + getMinimumDelayToServer() - (System.currentTimeMillis() - messageTime);
		return actualDelay;
	}
//	public static MulticastClient createClient (String serverHost, String clientName) {		
//		MulticastClient client = new AMulticastClient();
//		client.join(serverHost, clientName);
//		return client;
//	}
//	public static MulticastClient createClient (String[] args) {
//		if (args.length < 2) {
//			Message.fatalError("Please supply server host name and clientName as main argument");
//			return null;
//		}
//		return createClient(args[0], args[1]);		
//	}
//	public static void main (String[] args) {
//		if (args.length < 2)
//			Message.fatalError("Please supply server host name and clientName as main argument");
//		MessageSender client = new AMulticastClient(args);
//		client.toOthers("Hi, I am " + args[1]);		
//	}
@Override
public String[] getUserNames() throws RemoteException {
	String[] values = {};
	values = clients.values().toArray(values);		
	return values;
}
public void setClients(Map<MessageReceiver, String> theClients) throws RemoteException  {
	clients = theClients;
}
@Override
public Map<MessageReceiver, String> getClients()  {
	// TODO Auto-generated method stub
	return clients;
}
@Override
public void setSerializedMulticastGroups(SerializedMulticastGroups newVal) {
	serializedMulticastGroups = newVal;
}
@Override
public SerializedMulticastGroups getSerializedMulticastGroups() {
	return serializedMulticastGroups;
}

	

}
